# Copyright (c) 2022 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Module containing the entry point for the ros_license_toolkit CLI.
"""

import argparse
import os
import sys
import timeit
from typing import Optional, Sequence

from ros_license_toolkit.checks import (LicensesInCodeCheck,
                                        LicenseTagExistsCheck,
                                        LicenseTagIsInSpdxListCheck,
                                        LicenseTextExistsCheck)
from ros_license_toolkit.package import get_packages_in_path
from ros_license_toolkit.ui_elements import (FAILURE_STR, SUCCESS_STR,
                                             Verbosity, green, major_sep,
                                             minor_sep, red, rll_print_factory)


def main(args: Optional[Sequence[str]] = None) -> int:
    """Main entry point for the ros_license_toolkit CLI.

    :param args: the command line arguments, defaults to sys.argv[1:]
    :type args: Sequence[str], optional
    :return: the exit code, `os.EX_OK` if all checks passed, `os.EX_USAGE` if
        no packages were found, `os.EX_DATAERR` if any check failed
    :rtype: int
    """
    if not args:
        args = sys.argv[1:]
    parser = argparse.ArgumentParser(
        description='Checks ROS packages for correct license declaration.')
    parser.add_argument(
        'path', default='.',
        help='path to ROS2 package or repo containing packages')
    parser.add_argument(
        '-c', '--generate_copyright_file', action='store_true',
        default=False, help='generate a copyright file')
    parser.add_argument(
        '-v', '--verbose', dest='verbose', action='store_true',
        default=False, help='enable verbose output')
    parser.add_argument(
        '-q', '--quiet', dest='quiet', action='store_true',
        default=False, help='disable most output')
    parsed_args = parser.parse_args(args)

    # Determine the verbosity level
    if parsed_args.quiet:
        verbosity = Verbosity.QUIET
    elif parsed_args.verbose:
        verbosity = Verbosity.VERBOSE
    else:
        verbosity = Verbosity.NORMAL
    rll_print = rll_print_factory(verbosity)

    # Sanity check the path
    assert isinstance(parsed_args.path, str)
    assert os.path.exists(
        parsed_args.path), f'Path {parsed_args.path} does not exist.'

    # Get the packages in the path
    packages = get_packages_in_path(parsed_args.path)
    if not packages:
        rll_print(f'No packages found in {parsed_args.path}', Verbosity.QUIET)
        return os.EX_USAGE
    rll_print(
        f'Found {len(packages)} packages in {parsed_args.path}',
        Verbosity.QUIET)
    rll_print(
        f' Packages: {", ".join([p.name for p in packages])}',
        Verbosity.VERBOSE)
    rll_print(major_sep())

    # lets time the execution
    start = timeit.default_timer()

    # Check the packages
    results_per_package = {}
    for package in packages:
        rll_print(f'[{package.name}]')
        assert package.repo is not None, 'Package must be in a git repo.'
        rll_print(
            f'git hash of ({package.repo.get_path()}):'
            f' {package.repo.get_hash()}')
        checks_to_perform = [
            LicenseTagExistsCheck(),
            LicenseTagIsInSpdxListCheck(),
            LicenseTextExistsCheck(),
            LicensesInCodeCheck()]

        for check in checks_to_perform:
            check.check(package)
            rll_print(check)
            rll_print(check.verbose(), Verbosity.VERBOSE)

        if all(checks_to_perform):
            rll_print(minor_sep())
            rll_print(f"[{package.name}] Overall:" + green(
                f"\n {SUCCESS_STR}"))
            rll_print(major_sep())
            results_per_package[package.abspath] = True
        else:
            rll_print(minor_sep())
            rll_print(f"[{package.name}] Overall:" + red(
                f"\n {FAILURE_STR}"))
            rll_print(major_sep())
            results_per_package[package.abspath] = False
        if parsed_args.generate_copyright_file:
            package.write_copyright_file()

    stop = timeit.default_timer()
    rll_print(f'Execution time: {stop - start:.2f} seconds', Verbosity.QUIET)

    # Print the overall results
    if all(results_per_package.values()):
        rll_print("All packages:" + green(
            f"\n {SUCCESS_STR}"), Verbosity.QUIET)
        return os.EX_OK
    rll_print("All packages:" + red(f"\n {FAILURE_STR}"), Verbosity.QUIET)
    return os.EX_DATAERR



# Copyright (c) 2022 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Module containing the LicenseTag class and related functions to handle
license tags in package.xml files.
"""

import os
import xml.etree.ElementTree as ET
from glob import glob
from typing import List, Optional, Set

from spdx.config import LICENSE_MAP


def is_license_name_in_spdx_list(license_name: str) -> bool:
    """Check if a license name is in the SPDX list of licenses."""
    return license_name in LICENSE_MAP or \
        license_name in LICENSE_MAP.values()


def to_spdx_license_tag(license_name: str) -> str:
    """Convert a license name to a SPDX license tag
    (assuming it is shorter than the name).
    This is because the dict from spdx.config.LICENSE_MAP
    contains both pairings (tag, name) and (name, tag).
    """
    for tag, name in LICENSE_MAP.items():
        if license_name in [tag, name]:
            if len(tag) < len(name):
                return tag
            # else
            return name
    raise ValueError("License name not in SPDX list.")


def _eval_glob(glob_str: str, pkg_path: str) -> Set[str]:
    """Evaluate a glob string and return a set of matching relative paths."""
    paths = set()
    for fpath in glob(os.path.join(pkg_path, glob_str), recursive=True):
        if not os.path.isfile(fpath):
            continue
        paths.add(os.path.relpath(fpath, pkg_path))
    return paths


class LicenseTag:
    """A license tag found in a package.xml file."""

    def __init__(self, element: ET.Element, pkg_path: str):
        """Initialize a license tag from an XML element."""
        self.element = element
        assert self.element.text is not None, "License tag must have text."

        raw_license_name: str = str(self.element.text)
        # Name of the license (in SPDX tag format for comparability)
        try:
            self.id = to_spdx_license_tag(raw_license_name)
        except ValueError:
            # If the license name is not in the SPDX list,
            # we assume it is a custom license and use the name as-is.
            # This will be detected in `LicenseTagIsInSpdxListCheck`.
            self.id = raw_license_name

        # Path to the file containing the license text
        # (relative to package root)
        self.license_text_file: Optional[str] = element.attrib.get(
            "file", None)

        # Paths to the source files that are licensed under this license
        self._source_files: Optional[Set[str]] = None
        self.source_files_str: str = element.attrib.get("source-files", '')
        if not self.source_files_str:
            # If no source-files attribute is given, assume all files
            # are licensed under this license.
            self.source_files_str = "**"
        else:
            self._source_files = set()
            for src_glob in self.source_files_str.split(" "):
                self._source_files.update(_eval_glob(src_glob, pkg_path))

        # Path of package file this is in
        self.package_path: str = pkg_path

    def __str__(self) -> str:
        """Return a string representation of this license tag."""
        assert self.id is not None, "License must have a name."
        return self.id

    def has_license_text_file(self) -> bool:
        """Return whether this license tag has a file attribute."""
        return self.license_text_file is not None

    def has_source_files(self) -> bool:
        """Return whether this license tag has a source-files attribute."""
        return self._source_files is not None

    def get_license_id(self) -> str:
        """Return the license name."""
        assert self.id is not None, "License tag must have an id."
        return self.id

    def get_license_text_file(self) -> str:
        """Return the file attribute."""
        assert self.license_text_file is not None, \
            "License tag must have file attribute."
        return self.license_text_file

    @property
    def source_files(self) -> Set[str]:
        """Return the source-files attribute."""
        assert self._source_files is not None, \
            "License tag must have source-files attribute."
        return self._source_files

    def make_this_the_main_license(self, other_licenses: List["LicenseTag"]):
        """Make this the main license for the package."""
        assert not self.has_source_files(), \
            "This must not have a source-files, yet."
        assert self.source_files_str == "**", \
            "This must have a source-files attribute of '**'."
        source_files = _eval_glob(
            self.source_files_str, self.package_path)
        for other_license in other_licenses:
            if other_license == self:
                continue
            source_files -= other_license.source_files
        self._source_files = source_files



# Copyright (c) 2022 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains UI elements for the CLI.
"""

from enum import Enum, auto

# colors

GREEN = "\033[92m"
RED = "\033[91m"
NC = "\033[00m"


def red(message: str):
    """Make this `message` red"""
    return f"{RED}{message}{NC}"


def green(message: str):
    """Make this `message` green"""
    return f"{GREEN}{message}{NC}"

# further UI elements


FAILURE_STR = "FAILURE"
SUCCESS_STR = "SUCCESS"
NO_REASON_STR = "No reason provided."
N_SEP = 20


def major_sep() -> str:
    """Get a major separator"""
    return "=" * N_SEP


def minor_sep() -> str:
    """Get a minor separator"""
    return "-" * N_SEP


# verbosity levels

class Verbosity(Enum):
    """Verbosity levels for the output"""
    QUIET = auto()
    NORMAL = auto()
    VERBOSE = auto()


def rll_print_factory(verbosity: Verbosity):
    """Return a function that prints only if the verbosity is high enough"""
    def rll_print(message: str, level: Verbosity = Verbosity.NORMAL):
        """Print `message` if the verbosity is high enough"""
        if level.value <= verbosity.value:
            print(message)
    return rll_print



# Copyright (c) 2023 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Common utility functions."""

from typing import Any, Dict


def is_license_text_file(scan_results: Dict[str, Any]) -> bool:
    """Check if a file is a license text file."""
    for _license in scan_results["licenses"]:
        if _license["matched_rule"]["is_license_text"] and \
                _license["score"] >= 99:
            return True
    return False



# Copyright (c) 2023 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Assemble copyright notices for a package."""

import os
import re
from typing import Any, Dict, List, Tuple, Union

from scancode.api import get_copyrights


def _get_copyright_strs_from_results(
        scan_results: Dict[str, Any]) -> List[str]:
    """Get copyright strings from scan results."""
    return [cpr['copyright'] for cpr in scan_results['copyrights']]


def _get_year_from_copyright_str(cpr_str: str) -> Union[int, Tuple[int, int]]:
    """Get the year from a copyright string."""
    finds = re.findall(r"\d{4}", cpr_str)
    if len(finds) == 1:
        return int(finds[0])
    elif len(finds) == 2:
        return (int(finds[0]), int(finds[1]))
    else:
        raise ValueError("Unexpected number of digits in year")


class CopyrightPerFile:
    """A copyright notice for a single file."""

    def __init__(self, file_path: str, copyright_text: str):
        self.file_path = file_path
        for prefix_to_remove in [
            'copyright',
            'Copyright (c) ',
            'Copyright ',
            'Copyright'
        ]:
            if copyright_text.startswith(prefix_to_remove):
                copyright_text = copyright_text[len(prefix_to_remove):]
                break
        self.copyright_text = copyright_text

    def __str__(self):
        return self.copyright_text


class CopyrightPerPkg:
    def __init__(self, pkg):
        self.pkg = pkg
        # one section per license tag
        # each section is a list of unique copyright lines
        self.copyright_strings: Dict[str, List[str]] = {}
        for key, license_tag in self.pkg.license_tags.items():
            cprs = set()
            for source_file in license_tag.source_files:
                fpath = os.path.join(self.pkg.abspath, source_file)
                res = get_copyrights(fpath)
                if len(res) == 0:
                    continue
                for cpr in _get_copyright_strs_from_results(res):
                    cprs.add(CopyrightPerFile(source_file, cpr))
            self.copyright_strings[key] = sorted(
                {cpr.copyright_text for cpr in cprs})

    def __str__(self):
        return " ".join(" ".join(copyrights)
                        for copyrights in self.copyright_strings.values())



# Copyright (c) 2022 - see the NOTICE file in root or repo


# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains the Package class.
"""

import os
import xml.etree.ElementTree as ET
from typing import Any, Dict, List, Optional

from rospkg import RosPack, list_by_path
from rospkg.common import PACKAGE_FILE
from scancode.api import get_licenses

from ros_license_toolkit.common import is_license_text_file
from ros_license_toolkit.copyright import CopyrightPerPkg
from ros_license_toolkit.license_tag import LicenseTag
from ros_license_toolkit.repo import NotARepoError, Repo

# files we ignore in scan results
IGNORED_FILES = [
    "package.xml",
    "setup.py",
    "setup.cfg",
    "CMakeLists.txt"
]


def get_spdx_license_name(scan_results: Dict[str, Any]) -> Optional[str]:
    """Get the SPDX license name from scan results."""
    for _license in scan_results["licenses"]:
        if _license["score"] >= 99:
            return _license["spdx_license_key"]
    return None


class PackageException(Exception):
    """Exception raised when a package is invalid."""


class MoreThanOneLicenseWithoutSourceFilesTag(PackageException):
    """Exception raised when a package has more than one license tag without
    source files."""


class MoreThanOneLicenseWithoutLicenseTextFile(PackageException):
    """Exception raised when a package has more than one license tag without
    a license text file."""


class Package:
    """This represents a ros package, defined by its `path` (absolute) and
    results within it."""

    def __init__(self, path: str, repo: Optional[Repo] = None):
        # absolute path to this package
        self.abspath: str = path

        # relative path to the parent repo, if any
        self.repo: Optional[Repo] = repo

        # name of this package by its folder name
        self.name: str = os.path.basename(self.abspath)

        # Files found by scanner that contain license information
        # this is Optional, because it is only evaluated on the first call
        self._found_files_w_licenses: Optional[Dict[str, Any]] = None

        # Files that are found to contain license texts
        # this is Optional, because it is only evaluated on the first call
        self._found_license_texts: Optional[Dict[str, Any]] = None

        # The xml tree of the package.xml file
        # this is Optional, because it is only evaluated on the first call
        self._package_xml: Optional[ET.ElementTree] = None

        # The license tags found in the package.xml file
        # this is Optional, because it is only evaluated on the first call
        self._license_tags: Optional[Dict[str, LicenseTag]] = None

    def _get_path_relative_to_pkg(self, path: str) -> str:
        """Get path relative to pkg root"""
        return os.path.relpath(path, self.abspath)

    @property
    def found_files_w_licenses(self) -> Dict[str, Any]:
        """Get a dict of files in the package and their license scan results.
        Note that the code is only evaluated on the first call."""
        if self._found_files_w_licenses is None:
            self._run_scan_and_save_results()
        assert self._found_files_w_licenses is not None
        return self._found_files_w_licenses

    @property
    def found_license_texts(self) -> Dict[str, Any]:
        """Get a dict of files in the package and their license scan results.
        Note that the code is only evaluated on the first call."""
        if self._found_license_texts is None:
            self._run_scan_and_save_results()
        assert self._found_license_texts is not None
        return self._found_license_texts

    def _run_scan_and_save_results(self):
        """Get a dict of files in the package and their license scan results.
        Note that the code is only evaluated on the first call."""
        if not (self._found_files_w_licenses is None or (
                self._found_license_texts is None)):
            return self._found_files_w_licenses, self._found_license_texts
        self._found_files_w_licenses = {}
        self._found_license_texts = {}
        for (root, _, files) in os.walk(self.abspath):
            for fname in files:
                if fname in IGNORED_FILES:
                    continue
                # Path relative to cwd
                fpath = os.path.join(root, fname)
                # Path relative to package root
                fpath_rel_to_pkg = self._get_path_relative_to_pkg(fpath)
                scan_results = get_licenses(fpath)
                if is_license_text_file(scan_results):
                    self._found_license_texts[fpath_rel_to_pkg
                                              ] = scan_results
                else:
                    # not a license text file but also interesting
                    self._found_files_w_licenses[fpath_rel_to_pkg
                                                 ] = scan_results
        # look also in the repo for license text files
        if self.repo is not None:
            for path, res in self.repo.license_text_files.items():
                self._found_license_texts[
                    self._get_path_relative_to_pkg(path)] = res

    @property
    def package_xml(self):
        """Get xml of `package.xml` as `ElementTree` object."""
        if self._package_xml is None:
            self._package_xml = ET.parse(
                os.path.join(self.abspath, 'package.xml'))
        return self._package_xml

    @property
    def license_tags(self) -> Dict[str, LicenseTag]:
        """Get all license tags in the package.xml file."""
        if self._license_tags is not None:
            return self._license_tags
        self._license_tags = {}
        for license_tag in self.package_xml.iterfind('license'):
            tag = LicenseTag(license_tag, self.abspath)
            self._license_tags[tag.get_license_id()] = tag

        # One license tag can have no source-files attribute.
        # But there can be only one such tag.
        # This is then the main license.
        if len(list(filter(lambda x: not x.has_source_files(),
                           self._license_tags.values()))) > 1:
            raise MoreThanOneLicenseWithoutSourceFilesTag(
                "There must be at most one license tag without "
                "source-files.")
        for tag in self._license_tags.values():
            if not tag.has_source_files():
                tag.make_this_the_main_license(
                    list(self._license_tags.values()))
                break

        # One license tag can have no file attribute, but only one.
        # It may be associated to a license text file in the package or
        # the repo.
        if len(list(filter(lambda x: not x.has_license_text_file(),
                           self._license_tags.values()))) > 1:
            raise MoreThanOneLicenseWithoutLicenseTextFile(
                "There must be at most one license tag without "
                "a license text file.")
        for tag in self._license_tags.values():
            if not tag.has_license_text_file():
                license_texts = self.found_license_texts
                if len(license_texts) == 1:
                    tag.license_text_file = list(license_texts.keys())[0]
                else:
                    for license_text_file in license_texts:
                        if "LICENSE" in license_text_file:
                            tag.license_text_file = license_text_file
                            break
                break

        return self._license_tags

    def get_license_files(self) -> List[str]:
        """Get all license text files associated with license tags
        in the package.xml."""
        return [x.get_license_text_file() for x in
                self.license_tags.values()]

    def get_copyright_file_contents(self) -> str:
        """Get a string representation of the copyright notice."""
        copyright = CopyrightPerPkg(self)
        cpr_str = "".join((
            "Format: https://www.debian.org/doc/packaging-manuals/copyright",
            "-format/1.0/\n",
            "Source: tbd\n",
            f"Upstream-Name: {self.name}\n\n",))
        for key, cprs in copyright.copyright_strings.items():
            source_files_str = self.license_tags[key].source_files_str
            cpr_str += f"Files:\n {source_files_str}\n"
            cpr_str += "Copyright: "
            cpr_str += "\n           ".join(cprs)
            license = self.license_tags[key]
            cpr_str += f"\nLicense: {license.id}\n"
            assert license.license_text_file, \
                "License text file must be defined."
            with open(os.path.join(
                    self.abspath,
                    license.license_text_file)) as f:
                license_lines = f.readlines()
            for line in license_lines:
                cpr_str += f" {line}"
            cpr_str += "\n\n"
        return cpr_str

    def write_copyright_file(self):
        with open(os.path.join(
                self.abspath,
                'copyright'), 'w') as f:
            f.write(self.get_copyright_file_contents())


def get_packages_in_path(path: str) -> List[Package]:
    """Get all ROS packages in a given path."""
    packages = []
    try:
        repo: Optional[Repo] = Repo(os.path.abspath(path))
    except NotARepoError:
        repo = None
    for pkg in list_by_path(PACKAGE_FILE, path, {}):
        ros_pkg = RosPack([path]).get_path(pkg)
        packages.append(Package(ros_pkg, repo))
    return packages



# Copyright (c) 2022 - for information on the respective copyright owner
# see the NOTICE file and/or the repository
# https://github.com/boschresearch/ros_license_toolkit

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains the Repo class.
"""

import os
from typing import Any, Dict, Optional

import git
from scancode.api import get_licenses

from ros_license_toolkit.common import is_license_text_file

# how many folders up to search for a repo
REPO_SEARCH_DEPTH = 5


def is_git_repo(path: str) -> bool:
    """Check if a path is a git repo."""
    return os.path.isdir(os.path.join(path, ".git"))


class NotARepoError(Exception):
    """Exception raised when we can't find a repo."""


class Repo:
    """Represents a git repository."""

    def __init__(self, package_path: str):
        """Initialize a Repo object.

        :param package_path: Absolute path to the package.
        :type package_path: str
        """

        # absolute path to the package
        self.abs_package_path: str = package_path

        # repo path relative to the package
        relpath: Optional[str] = None
        search_path = package_path
        for _ in range(REPO_SEARCH_DEPTH + 1):
            if is_git_repo(search_path):
                relpath = os.path.relpath(
                    search_path, self.abs_package_path)
                break
            search_path = os.path.dirname(search_path)

        if relpath is None:
            raise NotARepoError("No git repo found for package.")

        # absolute path to the repo
        self.abs_path: str = os.path.normpath(os.path.join(
            self.abs_package_path, relpath))

        # (for logging purposes) the current git hash
        repo = git.Repo(search_path)
        self.git_hash: str = repo.head.object.hexsha

        # scan files in the repo
        self.license_text_files: Dict[str, Dict[str, Any]] = {}
        for file in os.scandir(self.abs_path):
            fpath = os.path.join(self.abs_path, file)
            if not os.path.isfile(fpath):
                continue
            scan_results = get_licenses(fpath)
            if is_license_text_file(scan_results):
                self.license_text_files[fpath] = scan_results

    def __eq__(self, __o) -> bool:
        """Check if two repos are the same."""
        return os.path.samefile(self.abs_path, __o.abs_path)

    def get_path(self) -> str:
        """Get the absolute path to the repo."""
        return self.abs_path

    def get_hash(self) -> str:
        """Get the current git hash."""
        return self.git_hash
